@@ -23,7 +23,14 @@
 /*
  * LDAP address book
  * NOTES:
- * ldap caching deleted since it is deprecated in openldap-2.1.x series.
+ * a) ldap caching deleted since it is deprecated in openldap-2.1.x series.
+ * b) In principle, only inetOrgPerson class may have mail attributes, see
+ * eg: http://www.andrew.cmu.edu/user/dd26/ldap.akbkhome.com/objectclass/organizationalPerson.html
+ *
+ * However, ActiveDirectory apparently gets it wrong (or its
+ * administrators, may be?) and all the objects are of
+ * organizationalPerson type. We are lenient.
+ *
  */
 #include "config.h"
 
@@ -45,7 +52,7 @@
 #include "address-book-ldap.h"
 #include "information.h"
 #include "libbalsa-conf.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static const int DEBUG_LDAP = 0;
 /* don't search when prefix has length shorter than LDAP_MIN_LEN */
@@ -338,6 +345,11 @@ libbalsa_address_book_ldap_open_connection(LibBalsaAddressBookLdap * ab)
     }
 #endif /* HAVE_CYRUS_SASL */
 
+    /* do not follow referrals (OpenLDAP binds anonymously here, which will usually
+     * fail */
+    if (result == LDAP_SUCCESS)
+	result = ldap_set_option(ab->directory, LDAP_OPT_REFERRALS, (void *)LDAP_OPT_OFF);
+
     if (result != LDAP_SUCCESS) {
         libbalsa_address_book_set_status(lbab,
                                          g_strdup(ldap_err2string(result)));
@@ -388,10 +400,13 @@ libbalsa_address_book_ldap_load(LibBalsaAddressBook * ab,
          */ 
         /* g_print("Performing full lookup...\n"); */
         ldap_filter = filter 
-            ? g_strdup_printf("(&(objectClass=inetOrgPerson)"
-                              "(|(cn=%s*)(sn=%s*)(mail=%s@*)))",
+            ? g_strdup_printf("(&(objectClass=organizationalPerson)(mail=*)"
+                              "(|(cn=%s*)(sn=%s*)(mail=%s*@*)))",
                               filter, filter, filter)
-            : g_strdup("(objectClass=inetOrgPerson)");
+            : g_strdup("(&(objectClass=organizationalPerson)(mail=*))");
+	if(DEBUG_LDAP)
+	    g_print("Send LDAP request: %s (basedn=%s)\n", ldap_filter,
+		    ldap_ab->base_dn);
         if(ldap_search_ext(ldap_ab->directory, ldap_ab->base_dn,
                            LDAP_SCOPE_SUBTREE, 
                            ldap_filter, book_attrs, 0, NULL, NULL,
@@ -589,6 +604,12 @@ libbalsa_address_book_ldap_add_address(LibBalsaAddressBook *ab,
 	    return LBABERR_CANNOT_CONNECT;
     }
 
+    if(ldap_ab->priv_book_dn == NULL) {
+        libbalsa_address_book_set_status
+            (ab, _("Undefined location of user address book"));
+        return LBABERR_CANNOT_WRITE;
+    }                                    
+
     dn = g_strdup_printf("mail=%s,%s",
                          (char*)address->address_list->data,
                          ldap_ab->priv_book_dn);
@@ -958,8 +979,8 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     if(new_prefix) *new_prefix = NULL;
     ldap = rfc_2254_escape(prefix);
 
-    filter = g_strdup_printf("(&(mail=*)"
-                             "(|(cn=%s*)(sn=%s*)(mail=%s@*)))",
+    filter = g_strdup_printf("(&(objectClass=organizationalPerson)(mail=*)"
+                             "(|(cn=%s*)(sn=%s*)(mail=%s*@*)))",
 			     ldap, ldap, ldap);
     g_free(ldap);
     result = NULL;
@@ -973,13 +994,15 @@ libbalsa_address_book_ldap_alias_complete(LibBalsaAddressBook * ab,
     g_free(filter);
     switch (rc) {
     case LDAP_SUCCESS:
-	for(e = ldap_first_entry(ldap_ab->directory, result);
-	    e != NULL; e = ldap_next_entry(ldap_ab->directory, e)) {
-	    addr = lbabl_get_internet_address(ldap_ab->directory, e);
-	    if(new_prefix && !*new_prefix) 
-		*new_prefix = internet_address_to_string(addr, FALSE);
-	    res = g_list_prepend(res, addr);
-	}
+    case LDAP_PARTIAL_RESULTS:
+	if (result)
+	    for(e = ldap_first_entry(ldap_ab->directory, result);
+		e != NULL; e = ldap_next_entry(ldap_ab->directory, e)) {
+		addr = lbabl_get_internet_address(ldap_ab->directory, e);
+		if(new_prefix && !*new_prefix) 
+		    *new_prefix = internet_address_to_string(addr, FALSE);
+		res = g_list_prepend(res, addr);
+	    }
     case LDAP_SIZELIMIT_EXCEEDED:
     case LDAP_TIMELIMIT_EXCEEDED:
 	/*
