@@ -23,8 +23,14 @@
 #ifndef   __LIBBALSA_HTML_H__
 # define  __LIBBALSA_HTML_H__
 
+#include "config.h"
+
+# if defined(HAVE_GTKHTML2)
+/* gtkhtml2 uses deprecated api */
+#  undef GTK_DISABLE_DEPRECATED
+# endif
+
 #  include <gtk/gtk.h>
-#  include <libgnomeprint/gnome-print.h>
 
 /* We need this enum even if we're not using GtkHtml. */
 typedef enum {
@@ -36,23 +42,33 @@ typedef enum {
 
 # ifdef HAVE_GTKHTML
 
-typedef void (*LibBalsaHTMLPrintCallback) (GtkWidget * widget,
-					   GnomePrintContext *
-					   print_context, gdouble x,
-					   gdouble y, gdouble width,
-					   gdouble height,
-					   gpointer user_data);
+typedef void (*LibBalsaHtmlCallback) (const gchar * uri);
 
 GtkWidget *libbalsa_html_new(const gchar * text, size_t len,
+			     const gchar * charset,
 			     gpointer message,
-			     GCallback link_clicked_cb);
+                             LibBalsaHtmlCallback hover_cb,
+                             LibBalsaHtmlCallback clicked_cb);
 void libbalsa_html_to_string(gchar ** text, size_t len);
 gboolean libbalsa_html_can_zoom(GtkWidget * widget);
 void libbalsa_html_zoom(GtkWidget * widget, gint in_out);
 gboolean libbalsa_html_can_select(GtkWidget * widget);
 void libbalsa_html_select_all(GtkWidget * widget);
 void libbalsa_html_copy(GtkWidget * widget);
+guint libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text,
+			   guint len);
+
+#if defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT)
+
+#  include <libgnomeprint/gnome-print.h>
+
 gboolean libbalsa_html_can_print(void);
+typedef void (*LibBalsaHTMLPrintCallback) (GtkWidget * widget,
+					   GnomePrintContext *
+					   print_context, gdouble x,
+					   gdouble y, gdouble width,
+					   gdouble height,
+					   gpointer user_data);
 void libbalsa_html_print(GtkWidget * widget,
 			 GnomePrintContext * print_context,
 			 gdouble header_height, gdouble footer_height,
@@ -64,8 +80,7 @@ gint libbalsa_html_print_get_pages_num(GtkWidget * widget,
 				       gdouble header_height,
 				       gdouble footer_height);
 
-guint libbalsa_html_filter(LibBalsaHTMLType html_type, gchar ** text,
-			   guint len);
+#endif /* defined(HAVE_GNOME) && !defined(HAVE_GTK_PRINT) */
 
 # endif				/* HAVE_GTKHTML */
 
