@@ -26,7 +26,11 @@
 #include "config.h"
 
 #include <gtk/gtk.h>
-#include <address.h>
+#include <gmime/internet-address.h>
+
+#if ENABLE_ESMTP
+#include "smtp-server.h"
+#endif
 
 #ifdef __cpluscplus
 extern "C" 
@@ -58,12 +62,13 @@ extern "C"
         
         gchar* identity_name;
         
-        LibBalsaAddress* address;
+        InternetAddress *ia;
         gchar* replyto;
         gchar* domain;
         gchar* bcc;
         gchar* reply_string;
         gchar* forward_string;
+        gboolean send_mp_alternative;
 
         gchar* signature_path;
         gboolean sig_executable;
@@ -72,12 +77,21 @@ extern "C"
         gboolean sig_whenreply;
         gboolean sig_separator;
         gboolean sig_prepend;
+        gchar *face;
+        gchar *x_face;
+	gboolean request_mdn;
 
 #ifdef HAVE_GPGME
 	gboolean gpg_sign;
 	gboolean gpg_encrypt;
+	gboolean always_trust;
+	gboolean warn_send_plain;
 	gint crypt_protocol;
+        gchar *force_key_id;
 #endif
+#if ENABLE_ESMTP
+	LibBalsaSmtpServer *smtp_server;
+#endif                          /* ENABLE_ESMTP */
     };
 
     struct _LibBalsaIdentityClass 
@@ -91,12 +105,13 @@ extern "C"
     GObject* libbalsa_identity_new_with_name(const gchar* ident_name);
     
     void libbalsa_identity_set_identity_name(LibBalsaIdentity*, const gchar*);
-    void libbalsa_identity_set_address(LibBalsaIdentity*, LibBalsaAddress*);
+    void libbalsa_identity_set_address(LibBalsaIdentity*, InternetAddress*);
     void libbalsa_identity_set_replyto(LibBalsaIdentity*, const gchar*);
     void libbalsa_identity_set_domain(LibBalsaIdentity*, const gchar*);
     void libbalsa_identity_set_bcc(LibBalsaIdentity*, const gchar*);
     void libbalsa_identity_set_reply_string(LibBalsaIdentity* , const gchar*);
     void libbalsa_identity_set_forward_string(LibBalsaIdentity*, const gchar*);
+    void libbalsa_identity_set_send_mp_alternative(LibBalsaIdentity*, gboolean);
     void libbalsa_identity_set_signature_path(LibBalsaIdentity*, const gchar*);
     void libbalsa_identity_set_sig_executable(LibBalsaIdentity*, gboolean);
     void libbalsa_identity_set_sig_sending(LibBalsaIdentity*, gboolean);
@@ -104,6 +119,13 @@ extern "C"
     void libbalsa_identity_set_sig_whenreply(LibBalsaIdentity*, gboolean);
     void libbalsa_identity_set_sig_separator(LibBalsaIdentity*, gboolean);
     void libbalsa_identity_set_sig_prepend(LibBalsaIdentity*, gboolean);
+    gchar* libbalsa_identity_get_signature(LibBalsaIdentity*, 
+                                           GtkWindow *parent);
+#if ENABLE_ESMTP
+    void libbalsa_identity_set_smtp_server(LibBalsaIdentity * ident,
+                                           LibBalsaSmtpServer *
+                                           smtp_server);
+#endif                          /* ENABLE_ESMTP */
 
 #ifdef HAVE_GPGME
     void libbalsa_identity_set_gpg_sign(LibBalsaIdentity*, gboolean);
@@ -114,6 +136,9 @@ extern "C"
     void libbalsa_identity_config_dialog(GtkWindow * parent,
                                          GList ** identities,
                                          LibBalsaIdentity ** current,
+#if ENABLE_ESMTP
+					 GSList * smtp_servers,
+#endif                          /* ENABLE_ESMTP */
                                          void (*changed_cb)(gpointer));
 
     typedef void (*LibBalsaIdentityCallback) (gpointer data,
@@ -125,8 +150,7 @@ extern "C"
                                          LibBalsaIdentityCallback update,
                                          gpointer data);
 
-    LibBalsaIdentity* libbalsa_identity_new_config(const gchar* prefix,
-						   const gchar* name);
+    LibBalsaIdentity* libbalsa_identity_new_config(const gchar* name);
     void libbalsa_identity_save(LibBalsaIdentity* id, const gchar* prefix);
 
 #ifdef __cplusplus
