@@ -31,12 +31,14 @@
 #include <auth-client.h>
 #endif
 
-#ifdef HAVE_PCRE
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
 typedef enum _LibBalsaCodeset LibBalsaCodeset;
 
@@ -84,23 +86,21 @@ struct _LibBalsaCodesetInfo {
 };
 
 typedef void (*libbalsa_url_cb_t) (GtkTextBuffer *, GtkTextIter *,
-				   const gchar *, gpointer);
+				   const gchar *, guint, gpointer);
 typedef struct _LibBalsaUrlInsertInfo LibBalsaUrlInsertInfo;
 struct _LibBalsaUrlInsertInfo {
     libbalsa_url_cb_t callback;
     gpointer callback_data;
     gboolean buffer_is_flowed;
-    gchar *ml_url;
     GString *ml_url_buffer;
 };
 
+#define LIBBALSA_ERROR_QUARK (g_quark_from_static_string("libbalsa"))
+
 extern LibBalsaCodesetInfo libbalsa_codeset_info[];
 GtkWidget *libbalsa_charset_button_new(void);
 LibBalsaTextAttribute libbalsa_text_attr_string(const gchar * string);
 LibBalsaTextAttribute libbalsa_text_attr_file(const gchar * filename);
-const gchar *libbalsa_file_get_charset(const gchar * filename);
-
-gchar *libbalsa_lookup_mime_type(const gchar * path);
 
 size_t libbalsa_readfile(FILE * fp, char **buf);
 size_t libbalsa_readfile_nostat(FILE * fp, char **buf);
@@ -122,7 +122,6 @@ gchar *libbalsa_wrap_rfc2646(gchar * par, gint width,
 void libbalsa_wrap_view(GtkTextView * view, gint length);
 void libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
                             gint lines);
-void libbalsa_prepare_delsp(GtkTextBuffer * buffer);
 
 const char* libbalsa_set_charset(const gchar * charset);
 const char* libbalsa_set_send_charset(const gchar * charset);
@@ -138,14 +137,20 @@ gboolean libbalsa_utf8_sanitize(gchar ** text, gboolean fallback,
 gboolean libbalsa_utf8_strstr(const gchar *s1,const gchar *s2);
 gboolean libbalsa_insert_with_url(GtkTextBuffer * buffer,
 				  const char *chars,
-				  const char *all_chars,
+				  guint len,
 				  GtkTextTag * tag,
 				  LibBalsaUrlInsertInfo *url_info);
+#if USE_GREGEX
+void libbalsa_unwrap_selection(GtkTextBuffer * buffer, GRegex * rex);
+gboolean libbalsa_match_regex(const gchar * line, GRegex * rex,
+			      guint * count, guint * index);
+#else                           /* USE_GREGEX */
 void libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex);
 gboolean libbalsa_match_regex(const gchar * line, regex_t * rex,
 			      guint * count, guint * index);
+#endif                          /* USE_GREGEX */
 
-int libbalsa_safe_open (const char *path, int flags, mode_t mode);
+int libbalsa_safe_open (const char *path, int flags, mode_t mode, GError **err);
 int libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout);
 int libbalsa_unlock_file (const char *path, int fd, int dot);
 int libbalsa_safe_rename (const char *src, const char *target);
@@ -178,4 +183,9 @@ gchar *libbalsa_guess_ldif_file(void);
 
 gboolean libbalsa_ldap_exists(const gchar *server);
 
+gboolean libbalsa_path_is_below_dir(const gchar * path, const gchar * dir);
+
+gchar * libbalsa_text_to_html(const gchar * title, const gchar * body, const gchar * lang);
+GString * libbalsa_html_encode_hyperlinks(GString * paragraph);
+
 #endif				/* __LIBBALSA_MISC_H__ */
