@@ -21,6 +21,8 @@
 
 #include "assistant_init.h"
 
+#include <stdlib.h>
+
 #include <glib/gi18n.h>
 #include "libbalsa-conf.h"
 #include "misc.h"
@@ -93,6 +95,10 @@ balsa_initdruid_apply(GtkAssistant * druid)
 
     g_signal_handlers_disconnect_by_func(G_OBJECT(druid),
                                          G_CALLBACK(exit), NULL);
+    libbalsa_conf_push_group("Notifications");
+    libbalsa_conf_set_bool("GtkUIManager", TRUE);
+    libbalsa_conf_set_bool("LibBalsaAddressView", TRUE);
+    libbalsa_conf_pop_group();
     config_save();
     gtk_main_quit();
 }
@@ -131,12 +137,14 @@ dismiss_the_wizard(GtkWidget *wizard)
 void
 balsa_init_begin(void)
 {
+    static const int ASSISTANT_WIDTH = 800;
+    static const int ASSISTANT_HEIGHT = 640;
     GtkWidget *assistant;
 
     assistant = gtk_assistant_new();
     gtk_window_set_title(GTK_WINDOW(assistant), _("Configure Balsa"));
     gtk_window_set_wmclass(GTK_WINDOW(assistant), "druid", "Balsa");
-    gtk_widget_set_size_request(assistant, 780, 580);
+    gtk_widget_set_size_request(assistant, ASSISTANT_WIDTH, ASSISTANT_HEIGHT);
 
     balsa_initdruid(GTK_ASSISTANT(assistant));
     gtk_widget_show_all(assistant);
