@@ -27,7 +27,7 @@
 #include <sys/stat.h>
 #include <string.h>
 
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "imap-server.h"
 #include "smtp-server.h"
 #include "server.h"
@@ -46,22 +46,6 @@ static void balsa_druid_page_user_next(GtkAssistant * druid,
                                        GtkWidget * page,
                                        BalsaDruidPageUser * user);
 
-static void 
-srv_changed_cb(GtkEditable *incoming_srv, gpointer data)
-{
-    BalsaDruidPageUser * user = (BalsaDruidPageUser*)data;
-    const gchar *prev_name;
-
-    prev_name = g_object_get_data(G_OBJECT(incoming_srv), "prev-name");
-    if(!prev_name ||
-       strcmp(prev_name, 
-              gtk_entry_get_text(GTK_ENTRY(user->account_name))) == 0) {
-        const gchar *aname = gtk_entry_get_text(GTK_ENTRY(incoming_srv));
-        gtk_entry_set_text(GTK_ENTRY(user->account_name), aname);
-        g_object_set_data_full(G_OBJECT(incoming_srv), "prev-name",
-                               g_strdup(aname), g_free);
-    }
-}
 
 static void
 balsa_druid_page_user_init(BalsaDruidPageUser * user,
@@ -73,11 +57,13 @@ balsa_druid_page_user_init(BalsaDruidPageUser * user,
            "(and you can find them later, if need be, in the Email "
            "application in the 'Preferences' and 'Identities' "
            "commands on the 'Tools' menu)");
+#if 0
     static const char *header21 =
         N_(" Whoever provides your email account should be able "
            "to give you the following information (if you have "
            "a Network Administrator, they may already have set "
            "this up for you):");
+#endif
     static const char* server_types[] = { "POP3", "IMAP", NULL };
     static const char* remember_passwd[] = {
         N_("Yes, remember it"), N_("No, type it in every time"), NULL };
@@ -100,16 +86,17 @@ balsa_druid_page_user_init(BalsaDruidPageUser * user,
 #endif
     label = GTK_LABEL(gtk_label_new(_(header2)));
     gtk_label_set_line_wrap(label, TRUE);
-    gtk_box_pack_start_defaults(GTK_BOX(page), GTK_WIDGET(label));
+    gtk_box_pack_start(GTK_BOX(page), GTK_WIDGET(label), TRUE, TRUE, 0);
 
     table = GTK_TABLE(gtk_table_new(10, 2, FALSE));
 
+#if 0
     label = GTK_LABEL(gtk_label_new(_(header21)));
     gtk_label_set_justify(label, GTK_JUSTIFY_CENTER);
     gtk_label_set_line_wrap(label, TRUE);
     gtk_table_attach(table, GTK_WIDGET(label), 0, 2, 0, 1,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 8, 4);
-
+#endif
     /* 2.1 */
     balsa_init_add_table_entry(table, row++,
                                _("Name of mail server for incoming _mail:"),
@@ -154,12 +141,8 @@ balsa_druid_page_user_init(BalsaDruidPageUser * user,
                                 _("_Remember your password:"),
                                 remember_passwd, druid,
                                 &(user->remember_passwd));
-    balsa_init_add_table_entry(table, row, _("_Refer to this account as:"),
-                               "",
-                               NULL, druid, page, &(user->account_name));
     gtk_table_set_row_spacing(table, row++, 10);
-    g_signal_connect(user->incoming_srv, "changed",
-                     (GCallback)srv_changed_cb, user);
+
 #if !defined(ENABLE_TOUCH_UI)
     preset = g_strconcat(g_get_home_dir(), "/mail", NULL);
     balsa_init_add_table_entry(table, row++, _("_Local mail directory:"),
@@ -260,7 +243,7 @@ static void
 balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
                            BalsaDruidPageUser * user)
 {
-    const gchar *host;
+    const gchar *host, *mailbox;
     gchar *uhoh;
     LibBalsaIdentity *ident;
 #if ENABLE_ESMTP
@@ -277,7 +260,6 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
     host = gtk_entry_get_text(GTK_ENTRY(user->incoming_srv));
     if(host && *host) {
         LibBalsaMailbox *mbx = NULL;
-        const gchar *name = gtk_entry_get_text(GTK_ENTRY(user->account_name));
         const gchar *login = gtk_entry_get_text(GTK_ENTRY(user->login));
         const gchar *passwd = gtk_entry_get_text(GTK_ENTRY(user->passwd));
         gboolean ssl = 
@@ -286,26 +268,31 @@ balsa_druid_page_user_next(GtkAssistant * druid, GtkWidget * page,
             balsa_option_get_active(user->remember_passwd) == 0;
         switch(balsa_option_get_active(user->incoming_type)) {
         case 0: /* POP */
-            mbx = create_pop3_mbx(name, host, ssl, login, passwd, remember);
+            mbx = create_pop3_mbx(host, host, ssl, login, passwd, remember);
             if(mbx)
                 config_mailbox_add(mbx, NULL);
             break;
         case 1: /* IMAP */
-            create_imap_mbx(name, host, ssl, login, passwd, remember);
+            create_imap_mbx(host, host, ssl, login, passwd, remember);
             break; 
         default: /* hm */;
         }
     }
 
     /* identity */
+
+    mailbox = gtk_entry_get_text(GTK_ENTRY(user->name));
     if (balsa_app.identities == NULL) {
-        ident = LIBBALSA_IDENTITY(libbalsa_identity_new());
+	gchar *domain = strrchr(mailbox, '@');
+        ident = LIBBALSA_IDENTITY(libbalsa_identity_new_with_name
+				  (_("Default Identity")));
         balsa_app.identities = g_list_append(NULL, ident);
+	if(domain)
+	    libbalsa_identity_set_domain(ident, domain+1);
     } else {
         ident = balsa_app.current_ident;
     }
-    internet_address_set_name(ident->ia,
-                              gtk_entry_get_text(GTK_ENTRY(user->name)));
+    internet_address_set_name(ident->ia, mailbox);
     internet_address_set_addr(ident->ia,
                               gtk_entry_get_text(GTK_ENTRY(user->email)));
 
