@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include <string.h>
+#include <stdlib.h>
 #ifdef BALSA_USE_THREADS
 /* _XOPEN_SOURCE is needed for rwlocks */
 #define _XOPEN_SOURCE 500
@@ -57,6 +58,8 @@ const gchar *pspell_suggest_modes[] = {
 };
 #endif                          /* HAVE_GTKSPELL */
 
+#define HIG_PADDING 12
+
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
 */
@@ -65,6 +68,13 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 {
     GtkWidget *dialog, *entry, *rememb;
     gchar *prompt, *passwd = NULL;
+#if defined(HAVE_GNOME_KEYRING)
+    static const gchar *remember_password_message =
+        N_("_Remember password in keyring");
+#else
+    static const gchar *remember_password_message =
+        N_("_Remember password");
+#endif
 
     g_return_val_if_fail(server != NULL, NULL);
     if (mbox)
@@ -74,8 +84,8 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
 			    mbox->name, server->user, server->host);
     else
 	prompt =
-	    g_strdup_printf(_("Mailbox _password for %s@%s:"), server->user,
-			    server->host);
+	    g_strdup_printf(_("_Password for %s@%s (%s):"), server->user,
+			    server->host, server->protocol);
 
     dialog = gtk_dialog_new_with_buttons(_("Password needed"),
                                          GTK_WINDOW(balsa_app.main_window),
@@ -83,6 +93,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          NULL); 
+    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox), HIG_PADDING);
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
                       gtk_label_new_with_mnemonic(prompt));
     g_free(prompt);
@@ -91,7 +102,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 20);
     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
 
-    rememb =  gtk_check_button_new_with_mnemonic(_("_Remember password"));
+    rememb =  gtk_check_button_new_with_mnemonic(_(remember_password_message));
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), rememb);
     if(server->remember_passwd)
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rememb), TRUE);
@@ -306,9 +317,9 @@ balsa_app_init(void)
 
     balsa_app.toolbar_wrap_button_text = TRUE;
     balsa_app.pwindow_option = WHILERETR;
-    balsa_app.wordwrap = TRUE;
+    balsa_app.wordwrap = FALSE; /* default to format=flowed. */
     balsa_app.wraplength = 72;
-    balsa_app.browse_wrap = TRUE;
+    balsa_app.browse_wrap = FALSE; /* GtkTextView will wrap for us. */
     balsa_app.browse_wrap_length = 79;
     balsa_app.shown_headers = HEADERS_SELECTED;
     balsa_app.show_all_headers = FALSE;
@@ -410,6 +421,9 @@ balsa_app_init(void)
 
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
+#if GTK_CHECK_VERSION(2, 10, 0)
+    balsa_app.notify_new_mail_icon = 1;
+#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
 
 #if !GTK_CHECK_VERSION(2, 11, 0)
     /* Tooltips */
@@ -437,18 +451,12 @@ void
 balsa_app_destroy(void)
 {
     config_views_save();
-    if (balsa_app.empty_trash_on_exit)
-	empty_trash(balsa_app.main_window);
-
     config_save();
 
     g_list_foreach(balsa_app.address_book_list, (GFunc)g_object_unref, NULL);
     g_list_free(balsa_app.address_book_list);
     balsa_app.address_book_list = NULL;
 
-    /* close all mailboxes */
-    gtk_widget_destroy(balsa_app.notebook);
-
     /* now free filters */
     g_slist_foreach(balsa_app.filters, (GFunc)libbalsa_filter_free, 
 		    GINT_TO_POINTER(TRUE));
@@ -885,3 +893,33 @@ balsa_find_index_by_mailbox(LibBalsaMailbox * mailbox)
     /* didn't find a matching mailbox */
     return NULL;
 }
+
+#if USE_GREGEX
+GRegex *
+balsa_quote_regex_new(void)
+{
+    static GRegex *regex  = NULL;
+    static gchar  *string = NULL;
+
+    if (string && strcmp(string, balsa_app.quote_regex) != 0) {
+        g_free(string);
+        string = NULL;
+        g_regex_unref(regex);
+        regex = NULL;
+    }
+
+    if (!regex) {
+        GError *err = NULL;
+
+        regex = g_regex_new(balsa_app.quote_regex, 0, 0, &err);
+        if (err) {
+            g_warning("quote regex compilation failed: %s", err->message);
+            g_error_free(err);
+            return NULL;
+        }
+        string = g_strdup(balsa_app.quote_regex);
+    }
+
+    return g_regex_ref(regex);
+}
+#endif                          /* USE_GREGEX */
