@@ -22,6 +22,7 @@
 #ifndef __BALSA_ICONS_H__
 #define __BALSA_ICONS_H__
 
+#include <gtk/gtk.h>
 #include "config.h"
 
 #define BALSA_PIXMAP_ATTACHMENT			"balsa_attachment"
@@ -37,6 +38,7 @@
 #define BALSA_PIXMAP_PREVIOUS			"balsa_previous"
 #define BALSA_PIXMAP_PREVIOUS_PART		"balsa_previous_part"
 #define BALSA_PIXMAP_POSTPONE			"balsa_postpone"
+#define BALSA_PIXMAP_REQUEST_MDN                "balsa_request_mdn"
 #define BALSA_PIXMAP_SEND			"balsa_send"
 #define BALSA_PIXMAP_SEND_RECEIVE		"balsa_send_receive"
 #define BALSA_PIXMAP_TRASH_EMPTY		"balsa_trash_empty"
@@ -86,6 +88,8 @@
 #define BALSA_PIXMAP_BOOK_BLUE                  "balsa_book_blue"
 #define BALSA_PIXMAP_BOOK_OPEN                  "balsa_book_open"
 
+#define BALSA_PIXMAP_DROP_DOWN                  "balsa_drop_down"
+
 void register_balsa_pixmaps(void);
 void register_balsa_pixbufs(GtkWidget * widget);
 void balsa_icon_create(const gchar ** data, GdkPixmap ** pmap,
