@@ -22,7 +22,6 @@
 #ifndef __BALSA_INDEX_H__
 #define __BALSA_INDEX_H__
 
-#include <gnome.h>
 #include "libbalsa.h"
 #include "filter.h"
 #include "mailbox-node.h"
@@ -31,13 +30,13 @@
 extern "C" {
 #endif				/* __cplusplus */
 
-    GtkType balsa_index_get_type(void);
+    GType balsa_index_get_type(void);
 
 #define BALSA_TYPE_INDEX          (balsa_index_get_type ())
-#define BALSA_INDEX(obj)          (GTK_CHECK_CAST (obj, BALSA_TYPE_INDEX, BalsaIndex))
-#define BALSA_INDEX_CLASS(klass)  (GTK_CHECK_CLASS_CAST (klass, BALSA_TYPE_INDEX, BalsaIndexClass))
-#define BALSA_IS_INDEX(obj)       (GTK_CHECK_TYPE (obj, BALSA_TYPE_INDEX))
-#define BALSA_IS_INDEX_CLASS(klass) (GTK_CHECK_CLASS_TYPE (klass, BALSA_TYPE_INDEX))
+#define BALSA_INDEX(obj)          (G_TYPE_CHECK_INSTANCE_CAST (obj, BALSA_TYPE_INDEX, BalsaIndex))
+#define BALSA_INDEX_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST (klass, BALSA_TYPE_INDEX, BalsaIndexClass))
+#define BALSA_IS_INDEX(obj)       (G_TYPE_CHECK_INSTANCE_TYPE (obj, BALSA_TYPE_INDEX))
+#define BALSA_IS_INDEX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE (klass, BALSA_TYPE_INDEX))
 
 
     typedef struct _BalsaIndex BalsaIndex;
@@ -46,8 +45,6 @@ extern "C" {
     struct _BalsaIndex {
         GtkTreeView tree_view;
         
-        GtkWidget* window;       
-
         /* the popup menu and some items we need to refer to */
         GtkWidget *popup_menu;
         GtkWidget *delete_item;
@@ -57,17 +54,20 @@ extern "C" {
         GtkWidget *move_to_item;
 
         BalsaMailboxNode* mailbox_node;
-        LibBalsaMessage* first_new_message;
-        LibBalsaMessage* current_message;
-	gboolean current_message_is_deleted;
-        gboolean prev_message;
-        gboolean next_message;
-	GArray *selected;
-        gchar *sos_filter; /* SenderOrSubject filter string, if any */
+        guint current_msgno;
+        guint next_msgno;
+	gboolean current_message_is_deleted:1;
+        gboolean prev_message:1;
+        gboolean next_message:1;
+        gboolean has_selection_changed_idle:1;
+        gboolean has_mailbox_changed_idle:1;
+        int    filter_no;
+        gchar *filter_string; /* Quick view filter string, if any */
 
         /* signal handler ids */
-        gulong selection_changed_id;
         gulong row_expanded_id;
+        gulong row_collapsed_id;
+        gulong selection_changed_id;
 
 	LibBalsaMailboxSearchIter *search_iter;
     };
@@ -106,9 +106,10 @@ extern "C" {
     void balsa_index_scroll_on_open(BalsaIndex *index);
     void balsa_index_update_tree(BalsaIndex *bindex, gboolean expand);
     void balsa_index_set_threading_type(BalsaIndex * bindex, int thtype);
-    void balsa_index_set_sos_filter(BalsaIndex *bindex,
-                                    const gchar *sos_filter,
-                                    LibBalsaCondition *flag_filter);
+    void balsa_index_set_view_filter(BalsaIndex *bindex,
+                                     int filter_no,
+                                     const gchar *filter_string,
+                                     LibBalsaCondition *filter);
 
 /* move or copy a list of messages */
     void balsa_index_transfer(BalsaIndex * index, GArray * msgnos,
@@ -116,26 +117,28 @@ extern "C" {
 
 /* select up/down the index */
     void balsa_index_select_next(BalsaIndex *);
-    void balsa_index_select_next_unread(BalsaIndex * bindex);
+    gboolean balsa_index_select_next_unread(BalsaIndex * index);
     void balsa_index_select_next_flagged(BalsaIndex * bindex);
     void balsa_index_select_previous(BalsaIndex *);
+    void balsa_index_set_next_msgno(BalsaIndex * bindex, guint msgno);
+    guint balsa_index_get_next_msgno(BalsaIndex * bindex);
 
     void balsa_index_find(BalsaIndex * bindex,
 			  LibBalsaMailboxSearchIter * search_iter,
                           gboolean previous, gboolean wrap);
 
 /* balsa index page stuff */
-    void balsa_message_reply(GtkWidget * widget, gpointer user_data);
-    void balsa_message_replytoall(GtkWidget * widget, gpointer user_data);
-    void balsa_message_replytogroup(GtkWidget * widget, gpointer user_data);
+    void balsa_message_reply(gpointer user_data);
+    void balsa_message_replytoall(gpointer user_data);
+    void balsa_message_replytogroup(gpointer user_data);
 
-    void balsa_message_forward_attached(GtkWidget * widget, gpointer data);
-    void balsa_message_forward_inline(GtkWidget * widget, gpointer data);
-    void balsa_message_forward_quoted(GtkWidget * widget, gpointer data);
-    void balsa_message_forward_default(GtkWidget * widget, gpointer data);
-    void balsa_message_continue(GtkWidget * widget, gpointer data);
+    void balsa_message_forward_attached(gpointer data);
+    void balsa_message_forward_inline(gpointer data);
+    void balsa_message_forward_quoted(gpointer data);
+    void balsa_message_forward_default(gpointer data);
+    void balsa_message_continue(gpointer data);
 
-    void balsa_message_move_to_trash(GtkWidget * widget, gpointer user_data);
+    void balsa_message_move_to_trash(gpointer user_data);
 
     void balsa_index_toggle_flag(BalsaIndex *index,
                                  LibBalsaMessageFlag flag);
@@ -144,6 +147,8 @@ extern "C" {
     gint balsa_find_notebook_page_num(LibBalsaMailbox * mailbox);
     void balsa_index_set_column_widths(BalsaIndex * index);
     GList * balsa_index_selected_list(BalsaIndex * index);
+    GArray * balsa_index_selected_msgnos_new(BalsaIndex * index);
+    void balsa_index_selected_msgnos_free(BalsaIndex * index, GArray * msgnos);
     void balsa_index_move_subtree(BalsaIndex * index,
                                   GtkTreePath * root,
                                   GtkTreePath * new_parent);
@@ -152,20 +157,27 @@ extern "C" {
     void balsa_index_refresh_date (BalsaIndex * index);
     void balsa_index_refresh_size (BalsaIndex * index);
 
-    /* Remove duplicate messages. */
-    void balsa_index_remove_duplicates(BalsaIndex * index);
-
     /* Expunge deleted messages. */
     void balsa_index_expunge(BalsaIndex * index);
-
-    /* Message window */
+ 
+    /* Message window */ 	 
     guint balsa_index_next_msgno(BalsaIndex * index, guint current_msgno);
     guint balsa_index_previous_msgno(BalsaIndex * index,
                                      guint current_msgno);
-    guint balsa_index_next_unread_msgno(BalsaIndex * index,
-                                        guint current_msgno);
-    guint balsa_index_next_flagged_msgno(BalsaIndex * index,
-                                         guint current_msgno);
+
+    /* Pipe messages */
+    void balsa_index_pipe(BalsaIndex * index);
+
+    /* Make sure messages are visible. */
+    void balsa_index_ensure_visible(BalsaIndex * index);
+
+    /* Select all without previewing any. */
+    void balsa_index_select_all(BalsaIndex * bindex);
+
+    /* Count of selected messages. */
+    gint balsa_index_count_selected_messages(BalsaIndex * bindex);
+
+#define BALSA_INDEX_VIEW_ON_OPEN "balsa-index-view-on-open"
 
 #ifdef __cplusplus
 }
