@@ -22,7 +22,8 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "config.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
+#include "balsa-icons.h"
 #include "balsa-print-object.h"
 #include "balsa-print-object-decor.h"
 #include "balsa-print-object-default.h"
@@ -131,7 +132,11 @@ balsa_print_object_text_plain(GList *list, GtkPrintContext * context,
 			      LibBalsaMessageBody * body,
 			      BalsaPrintSetup * psetup)
 {
+#if USE_GREGEX
+    GRegex *rex;
+#else                           /* USE_GREGEX */
     regex_t rex;
+#endif                          /* USE_GREGEX */
     gchar *textbuf;
     PangoFontDescription *font;
     gdouble c_at_x;
@@ -142,7 +147,11 @@ balsa_print_object_text_plain(GList *list, GtkPrintContext * context,
     gint par_len;
 
     /* set up the regular expression for qouted text */
+#if USE_GREGEX
+    if (!(rex = balsa_quote_regex_new()))
+#else                           /* USE_GREGEX */
     if (regcomp(&rex, balsa_app.quote_regex, REG_EXTENDED) != 0)
+#endif                          /* USE_GREGEX */
 	return balsa_print_object_default(list, context, body, psetup);
 
     /* start on new page if less than 2 lines can be printed */
@@ -209,8 +218,14 @@ balsa_print_object_text_plain(GList *list, GtkPrintContext * context,
 	    thispar = g_strndup(par_start, par_len);
 
 	    /* get the cite level and strip off the prefix */
+#if USE_GREGEX
 	    if (libbalsa_match_regex
-		(thispar, &rex, &cite_level, &cite_idx)) {
+		(thispar, rex, &cite_level, &cite_idx))
+#else                           /* USE_GREGEX */
+	    if (libbalsa_match_regex
+		(thispar, &rex, &cite_level, &cite_idx))
+#endif                          /* USE_GREGEX */
+            {
 		gchar *new;
 
 		new = thispar + cite_idx;
@@ -315,6 +330,9 @@ balsa_print_object_text_plain(GList *list, GtkPrintContext * context,
     /* clean up */
     pango_font_description_free(font);
     g_free(textbuf);
+#if USE_GREGEX
+    g_regex_unref(rex);
+#endif                          /* USE_GREGEX */
     return list;
 }
 
@@ -405,9 +423,6 @@ balsa_print_object_text(GList *list, GtkPrintContext * context,
 
 /* note: a vcard is an icon plus a series of labels/text, so this function actually
  * returns a BalsaPrintObjectDefault... */
-#ifdef HAVE_GNOME
-#include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#endif
 
 #define ADD_VCARD_FIELD(buf, labwidth, layout, field, descr)		\
     do {								\
@@ -434,7 +449,7 @@ balsa_print_object_text_vcard(GList * list,
     PangoTabArray *tabs;
     GString *desc_buf;
     gdouble c_max_height;
-    LibBalsaAddress * addr;
+    LibBalsaAddress * addr = NULL;
     gchar *textbuf;
 
     /* check if we can create an address from the body and fall back to default if 
@@ -443,10 +458,11 @@ balsa_print_object_text_vcard(GList * list,
 	textbuf = g_strdup(body->buffer);
     else
 	libbalsa_message_body_get_content(body, &textbuf, NULL);
-    addr = libbalsa_address_new_from_vcard(textbuf, body->charset);
+    if (textbuf)
+        addr = libbalsa_address_new_from_vcard(textbuf, body->charset);
     if (!addr) {
 	g_free(textbuf);
-	return balsa_print_object_default(list, context, body, psetup);
+	return balsa_print_object_text(list, context, body, psetup);
     }
 
     /* proceed with the address information */
@@ -462,7 +478,7 @@ balsa_print_object_text_vcard(GList * list,
     /* get the stock contacts icon or the mime type icon on fail */
     pod->pixbuf =
 	gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-				 "stock_contact", 48,
+				 BALSA_PIXMAP_IDENTITY, 48,
 				 GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
     if (!pod->pixbuf) {
 	gchar *conttype = libbalsa_message_body_get_mime_type(body);
