@@ -21,11 +21,11 @@
 
 #include "config.h"
 
-#include <gnome.h>
 #include "balsa-app.h"
 #include "filter-edit.h"
 #include "filter-funcs.h"
 #include "message.h"
+#include <glib/gi18n.h>
 
 #define FILTER_EDIT_ENTRY_MAX_LENGTH 256
 #define FILTER_EDIT_PADDING 6
@@ -84,7 +84,6 @@ GtkWidget * fe_condition_delete_button,* fe_condition_edit_button;
 
 /* ******************************** */
 
-#if GTK_CHECK_VERSION(2, 4, 0)
 option_list fe_search_type[] = {
     {N_("Simple"), CONDITION_STRING},
     {N_("Regular Expression"), CONDITION_REGEX},
@@ -104,27 +103,6 @@ option_list fe_op_codes[] = {
     {N_("OR"), FILTER_OP_OR},
     {N_("AND"), FILTER_OP_AND}
 };
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-option_list fe_search_type[] = {
-    {N_("Simple"), CONDITION_STRING, NULL},
-    {N_("Regular Expression"), CONDITION_REGEX, NULL},
-    {N_("Date interval"),CONDITION_DATE,NULL},
-    {N_("Flag condition"),CONDITION_FLAG,NULL}
-};
-
-option_list fe_actions[] = {
-    {N_("Copy to folder:"), FILTER_COPY, NULL},
-    {N_("Move to folder:"), FILTER_MOVE, NULL},
-    {N_("Print on printer:"), FILTER_PRINT, NULL},
-    {N_("Run program:"), FILTER_RUN, NULL},
-    {N_("Send to Trash"), FILTER_TRASH, NULL}
-};
-
-option_list fe_op_codes[] = {
-    {N_("OR"), FILTER_OP_OR, NULL},
-    {N_("AND"), FILTER_OP_AND, NULL}
-};
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 
 /* ******************************** */
 void
@@ -141,25 +119,24 @@ fe_enable_right_page(gboolean enabled)
  * Arguments:
  *    option_list[] options - array of options
  *    gint num - number of options
- *    GCallback func - callback function (will get data of i)
+ *    GCallback func - callback function
+ *    gpointer cb_data - user-data for func
  *
  * Returns:
  *    GtkOptionMenu - the menu created
  */
 
-#if GTK_CHECK_VERSION(2, 4, 0)
 static void
 fe_combo_box_info_free(struct fe_combo_box_info * info)
 {
     g_slist_free(info->values);
     g_free(info);
 }
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 
 GtkWidget *
-fe_build_option_menu(option_list options[], gint num, GCallback func)
+fe_build_option_menu(option_list options[], gint num, GCallback func,
+                     gpointer cb_data)
 {
-#if GTK_CHECK_VERSION(2, 4, 0)
     GtkWidget *combo_box;
     struct fe_combo_box_info *info;
     int i;
@@ -179,40 +156,11 @@ fe_build_option_menu(option_list options[], gint num, GCallback func)
     }
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), 0);
     if (func)
-	g_signal_connect(G_OBJECT(combo_box), "changed", func, NULL);
+	g_signal_connect(G_OBJECT(combo_box), "changed", func, cb_data);
     g_object_set_data_full(G_OBJECT(combo_box), BALSA_FE_COMBO_BOX_INFO,
                            info, (GDestroyNotify) fe_combo_box_info_free);
 
     return combo_box;
-#else /* GTK_CHECK_VERSION(2, 4, 0) */
-    GtkWidget *option_menu;
-    GtkWidget *menu;
-    int i;
-
-    if (num < 1)
-	return (NULL);
-
-    menu = gtk_menu_new();
-
-    for (i = 0; i < num; i++) {
-	options[i].widget =
-	    gtk_menu_item_new_with_label(_(options[i].text));
-	g_object_set_data(G_OBJECT(options[i].widget), "value",
-			  GINT_TO_POINTER(options[i].value));
-
-	gtk_menu_shell_append(GTK_MENU_SHELL(menu), options[i].widget);
-	if (func)
-	    g_signal_connect(G_OBJECT(options[i].widget), "activate",
-			     func, GINT_TO_POINTER(options[i].value));
-	gtk_widget_show(options[i].widget);
-    }
-
-    option_menu = gtk_option_menu_new();
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(option_menu), menu);
-    gtk_option_menu_set_history(GTK_OPTION_MENU(option_menu), 0);
-
-    return (option_menu);
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
 }				/* end fe_build_option_menu */
 
 /*
@@ -271,7 +219,7 @@ build_left_side(void)
     gtk_container_add(GTK_CONTAINER(bbox), fe_new_button);
     /* delete button */
     fe_delete_button =
-        balsa_stock_button_with_label(GNOME_STOCK_TRASH, _("_Delete"));
+        balsa_stock_button_with_label(GTK_STOCK_DELETE, _("_Delete"));
     g_signal_connect(G_OBJECT(fe_delete_button), "clicked",
 		     G_CALLBACK(fe_delete_pressed), NULL);
     gtk_container_add(GTK_CONTAINER(bbox), fe_delete_button);
@@ -310,6 +258,8 @@ build_match_page()
 		     2, 10, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     gtk_label_set_mnemonic_widget(GTK_LABEL(fe_name_label), fe_name_entry);
+    g_signal_connect(fe_name_entry, "changed",
+                     G_CALLBACK(fe_action_changed), NULL);
 
     /* The filter op-code : "OR" or "AND" all the conditions */ 
 
@@ -327,7 +277,9 @@ build_match_page()
 
     fe_op_codes_option_menu = fe_build_option_menu(fe_op_codes,
 						ELEMENTS(fe_op_codes),
-						NULL);
+						NULL, NULL);
+    g_signal_connect(fe_op_codes_option_menu, "changed",
+                     G_CALLBACK(fe_action_changed), NULL);
     gtk_box_pack_start(GTK_BOX(box), fe_op_codes_option_menu, FALSE, FALSE,
 		       2);
 
@@ -386,6 +338,9 @@ build_action_page(GtkWindow * window)
 {
     GtkWidget *page, *frame, *table;
     GtkWidget *box;
+#if GTK_CHECK_VERSION(2, 6, 0)
+    GtkWidget *dialog;
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
     page = gtk_vbox_new(TRUE, 5);
 
@@ -406,8 +361,23 @@ build_action_page(GtkWindow * window)
 		     0, 1, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
 
+#if GTK_CHECK_VERSION(2, 6, 0)
+    dialog =
+        gtk_file_chooser_dialog_new(_("Use Sound..."), NULL,
+                                    GTK_FILE_CHOOSER_ACTION_OPEN,
+                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                    NULL);
+    gtk_dialog_set_default_response(GTK_DIALOG(dialog),
+                                    GTK_RESPONSE_ACCEPT);
+    gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog),
+                                            GTK_RESPONSE_ACCEPT,
+                                            GTK_RESPONSE_CANCEL, -1);
+    fe_sound_entry = gtk_file_chooser_button_new_with_dialog(dialog);
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     fe_sound_entry =
 	gnome_file_entry_new("filter_sounds", _("Use Sound..."));
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
     gtk_table_attach(GTK_TABLE(table), fe_sound_entry, 1, 2, 0, 1,
 		     GTK_FILL | GTK_SHRINK | GTK_EXPAND, GTK_SHRINK, 5, 5);
     /* fe_sound_entry is initially sensitive, so to be consistent 
@@ -415,10 +385,15 @@ build_action_page(GtkWindow * window)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fe_sound_button), TRUE);
     g_signal_connect(G_OBJECT(fe_sound_button), "toggled",
                      G_CALLBACK(fe_button_toggled), fe_sound_entry);
+#if GTK_CHECK_VERSION(2, 6, 0)
+    g_signal_connect(G_OBJECT(dialog), "response",
+                     G_CALLBACK(fe_sound_response), NULL);
+#else /* GTK_CHECK_VERSION(2, 6, 0) */
     g_signal_connect(G_OBJECT
                      (gnome_file_entry_gtk_entry
                       (GNOME_FILE_ENTRY(fe_sound_entry))), "changed",
                      G_CALLBACK(fe_action_changed), NULL);
+#endif /* GTK_CHECK_VERSION(2, 6, 0) */
 
     fe_popup_button = gtk_check_button_new_with_label(_("Popup text:"));
     gtk_table_attach(GTK_TABLE(table),
@@ -452,7 +427,7 @@ build_action_page(GtkWindow * window)
 
     fe_action_option_menu =
         fe_build_option_menu(fe_actions, ELEMENTS(fe_actions),
-                             G_CALLBACK(fe_action_selected));
+                             G_CALLBACK(fe_action_selected), NULL);
     gtk_box_pack_start(GTK_BOX(box), fe_action_option_menu,
                        TRUE, FALSE, 1);
 
@@ -521,6 +496,8 @@ build_right_side(GtkWindow * window)
 static void
 fe_collect_user_headers(LibBalsaCondition * condition)
 {
+    g_return_if_fail(condition != NULL);
+
     switch (condition->type) {
     case CONDITION_STRING:
         if (CONDITION_CHKMATCH(condition, CONDITION_MATCH_US_HEAD)) {
@@ -585,6 +562,8 @@ filters_edit_dialog(void)
 	             G_CALLBACK(fe_destroy_window_cb), NULL);
 
     gtk_window_set_wmclass(GTK_WINDOW (fe_window), "filter-edit", "Balsa");
+    gtk_dialog_set_response_sensitive(GTK_DIALOG(fe_window),
+                                      GTK_RESPONSE_OK, FALSE);
 
     /* main hbox */
     hbox = gtk_hbox_new(FALSE, FILTER_EDIT_PADDING);
@@ -632,7 +611,7 @@ filters_edit_dialog(void)
 	/* Copy conditions */
         if (fil->condition)
             /* Copy conditions */
-            cpfil->condition = libbalsa_condition_clone(fil->condition);
+            cpfil->condition = libbalsa_condition_ref(fil->condition);
         else
             balsa_information(LIBBALSA_INFORMATION_WARNING,
                               _("Filter \"%s\" has no condition."),
@@ -649,10 +628,6 @@ filters_edit_dialog(void)
                            0, cpfil->name, 1, cpfil, -1);
     }
 
-#if !GTK_CHECK_VERSION(2, 4, 0)
-    /* To make sure we have at least one item in the combo list */
-    fe_add_new_user_header("X-Mailer");
-#endif /* GTK_CHECK_VERSION(2, 4, 0) */
     if (filter_errno!=FILTER_NOERR) {
 	filter_perror(filter_strerror(filter_errno));
 	gtk_widget_destroy(GTK_WIDGET(fe_window));
