@@ -87,6 +87,7 @@ extern "C" {
         SendmsgState state;
         gulong identities_changed_id;
 	gboolean flow;          /* send format=flowed */ 
+	gboolean send_mp_alt;   /* send multipart/alternative (plain and html) */ 
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
@@ -128,7 +129,8 @@ extern "C" {
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
                                   const gchar* val);
 
-    gboolean add_attachment(BalsaSendmsg * bsmsg, char *filename, 
+    gboolean add_attachment(BalsaSendmsg * bsmsg,
+                            const gchar *filename, 
                             gboolean is_a_tmp_file, 
                             const gchar *forced_mime_type);
 
@@ -139,8 +141,8 @@ extern "C" {
     BalsaSendmsg *sendmsg_window_new_from_list(LibBalsaMailbox * mailbox,
                                                GArray * selected,
                                                SendType type);
-    BalsaToolbarModel *sendmsg_window_get_toolbar_model(GtkUIManager **
-                                                        ui_manager);
+    BalsaToolbarModel *sendmsg_window_get_toolbar_model(void);
+    GtkUIManager *sendmsg_window_ui_manager_new(BalsaSendmsg * bsmsg);
 
 #define SENDMSG_WINDOW_QUIT_ON_CLOSE(bsmsg) ((bsmsg)->quit_on_close=TRUE)
 
